package com.devicehive.messages.handler.command;

/*
 * #%L
 * DeviceHive Backend Logic
 * %%
 * Copyright (C) 2016 DataArt
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */

import com.devicehive.eventbus.EventBus;
import com.devicehive.model.DeviceCommand;
import com.devicehive.model.eventbus.events.CommandsUpdateEvent;
import com.devicehive.model.rpc.CommandsUpdateRequest;
import com.devicehive.service.cache.command.CommandCacheService;
import com.devicehive.shim.api.Request;
import com.devicehive.shim.api.Response;
import com.devicehive.shim.api.server.RequestHandler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class CommandsUpdateHandler implements RequestHandler {

    private CommandCacheService commandCacheService;
    private EventBus eventBus;

    @Autowired
    public void setCommandCacheService(CommandCacheService commandCacheService) {
        this.commandCacheService = commandCacheService;
    }

    @Autowired
    public void setEventBus(EventBus eventBus) {
        this.eventBus = eventBus;
    }

    @Override
    public Response handle(Request request) {
        final DeviceCommand command = request.getBody().cast(CommandsUpdateRequest.class).getDeviceCommand();
        commandCacheService.store(command);

        eventBus.publish(new CommandsUpdateEvent(command));

        return Response.newBuilder().buildSuccess();
    }
}
