package com.devicehive.json.strategies;

/*
 * #%L
 * DeviceHive Common Dao Interfaces
 * %%
 * Copyright (C) 2016 DataArt
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */


import com.google.gson.ExclusionStrategy;
import com.google.gson.FieldAttributes;

import javax.validation.constraints.NotNull;

public class AnnotatedStrategy implements ExclusionStrategy {

    private final JsonPolicyDef.Policy policy;

    public AnnotatedStrategy(@NotNull JsonPolicyDef.Policy policy) {
        this.policy = policy;
    }

    @Override
    public boolean shouldSkipField(FieldAttributes f) {
        JsonPolicyDef policyAnnotation = f.getAnnotation(JsonPolicyDef.class);
        if (policyAnnotation == null) {
            // no policy annotation - filed should be skipped
            return true;
        }
        for (JsonPolicyDef.Policy definedPolicy : policyAnnotation.value()) {
            if (definedPolicy == policy) {
                // policy is found - field is to be included
                return false;
            }
        }
        return true;
    }

    @Override
    public boolean shouldSkipClass(Class<?> clazz) {
        return false;
    }
}
